 1
  
#include <Arduino.h>
#include <AccelStepper.h>

#define TURN 3200
#define YAW_DEG_PER_STEP 89.87
#define PTCH_DEG_PER_STEP 4000

#define LAT 51.592365 * DEG_TO_RAD
#define TIME_OFFSET 45.960804 / 15
#define MS_IN_HOUR 3600000.0

AccelStepper yaw = AccelStepper(1, 6, 3);
AccelStepper pitch = AccelStepper(1, 5, 2);

void run();
void home();
void calc_sun_pos();

int day;
bool sun_tracking;
uint64_t lastCalc;
float azimuth, elevation, timeDiff;

void setup()
{
    Serial.begin(9600);

    pinMode(9, INPUT_PULLUP);  //  
    pinMode(10, INPUT_PULLUP); //  

    yaw.setMaxSpeed(TURN / 3);
    pitch.setMaxSpeed(TURN * 2);

    yaw.setAcceleration(640);
    pitch.setAcceleration(3200);
    home();
}

void loop()
{
    if (Serial.available() > 0)
    {
        String data = Serial.readStringUntil('\n');
        if (data == "u")
        {
            pitch.move(TURN);
            yaw.stop();
            sun_tracking = false;
        }
        else if (data == "d")
        {
            pitch.move(-TURN);
            yaw.stop();
            sun_tracking = false;
        }
        else if (data == "l")
        {
            yaw.move(-TURN / 6);
            pitch.stop();
            sun_tracking = false;
        }
        else if (data == "r")
        {
            yaw.move(TURN / 6);
            pitch.stop();
            sun_tracking = false;
        }
        else if (data == "home")
        {
            home();
            sun_tracking = false;
        }
        else if (data == "auto")
            sun_tracking = true;
        else
        {
            byte spaceIndex = data.indexOf(' ');
            timeDiff = data.substring(0, spaceIndex).toFloat() - millis() / MS_IN_HOUR;
            day = data.substring(spaceIndex + 1).toInt();
        }
    }
    if (millis() / MS_IN_HOUR + timeDiff >= 24)
    {
        timeDiff -= 24;
        day++;
    }
    if ((millis() - lastCalc >= 30000) && sun_tracking)
    {
        calc_sun_pos();
        yaw.moveTo(constrain(azimuth, 0, 200) * YAW_DEG_PER_STEP);
        pitch.moveTo(constrain(elevation / 9, 0, 20) * PTCH_DEG_PER_STEP);
        lastCalc = millis();
    }
    run();
}

void run()
{
    if (yaw.distanceToGo() > 0)
    {
        yaw.enableOutputs();
        yaw.run();
    }
    else
        yaw.disableOutputs();

    if (pitch.distanceToGo() > 0)
    {
        pitch.enableOutputs();
        pitch.run();
    }
    else
        pitch.disableOutputs();
}

void calc_sun_pos()
{
    float d = -23.45 * cos(2 * PI / 365 * (day + 13)) * DEG_TO_RAD;
    float t = 15 * (millis() / MS_IN_HOUR + timeDiff + TIME_OFFSET - 12) * DEG_TO_RAD;
    float alpha = asin(sin(d) * sin(LAT) + cos(d) * cos(LAT) * cos(t));
    float gamma = acos((sin(d) * cos(LAT) - cos(d) * sin(LAT) * cos(t)) / cos(alpha)) * RAD_TO_DEG;
    azimuth = (t < 0) ? gamma : 360 - gamma;
    elevation = alpha * RAD_TO_DEG;

    Serial.print(elevation, 3);
    Serial.print('\t');
    Serial.print(azimuth, 3);
    Serial.print('\r');
}

void home()
{
    yaw.setSpeed(-TURN / 3);
    while (digitalRead(9))
        yaw.runSpeed();
    yaw.stop();
    yaw.setCurrentPosition(0);

    pitch.setSpeed(-TURN * 2);
    while (digitalRead(10))
        pitch.runSpeed();
    pitch.stop();
    pitch.setCurrentPosition(0);
}


 2
       
   
package com.rotatingplate

import android.bluetooth.BluetoothDevice
import android.bluetooth.BluetoothManager
import android.bluetooth.BluetoothSocket
import android.os.Bundle
import android.widget.Button
import android.widget.TableLayout
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import java.io.IOException
import java.time.LocalDateTime
import java.time.ZoneOffset
import java.util.*
import kotlin.math.roundToInt

class MainActivity : AppCompatActivity() {
    private lateinit var upButton: Button
    private lateinit var downButton: Button
    private lateinit var leftButton: Button
    private lateinit var rightButton: Button
    private lateinit var controls: TableLayout
    private lateinit var connectButton: Button
    private lateinit var autoButton: Button
    private lateinit var homeButton: Button
    private lateinit var bluetoothManager: BluetoothManager
    private lateinit var device: BluetoothDevice
    private lateinit var socket: BluetoothSocket
    private var animationDuration: Long = 0
    private var connectThread = Thread {}

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        upButton = findViewById(R.id.upButton)
        downButton = findViewById(R.id.downButton)
        leftButton = findViewById(R.id.leftButton)
        rightButton = findViewById(R.id.rightButton)
        controls = findViewById(R.id.controls)
        connectButton = findViewById(R.id.connectButton)
        autoButton = findViewById(R.id.autoButton)
        homeButton = findViewById(R.id.homeButton)
        animationDuration = resources.getInteger(android.R.integer.config_longAnimTime).toLong()

        connectButton.setOnClickListener {
            if (socket.isConnected) {
                socket.close()
                connectButton.text = getString(R.string.connect)
                showToast(getString(R.string.disconnected))
            } else {
                try {
                    showToast(getString(R.string.connecting))
                    bluetoothManager.adapter.enable()
                    while (true) if (bluetoothManager.adapter.isEnabled) break
                    socket =
                        device.createRfcommSocketToServiceRecord(UUID.fromString("00001101-0000-1000-8000-00805F9B34FB"))
                    if (connectThread.isAlive) connectThread.interrupt()
                    connectThread = Thread {
                        try {
                            socket.connect()
                            showToast(getString(R.string.connected))
                            runOnUiThread { connectButton.text = getString(R.string.disconnect) }
                        } catch (e: IOException) {
                            showToast(getString(R.string.io_error))
                        }
                    }
                    connectThread.start()
                } catch (e: SecurityException) {
                    showToast(getString(R.string.no_permissions))
                }
            }
        }
        autoButton.setOnClickListener {
            val moment = LocalDateTime.now(ZoneOffset.UTC)
            val out =
                (((moment.hour + moment.minute / 60.0) * 1000).roundToInt() / 1000.0).toString() + ' ' + moment.dayOfYear.toString()
            send("auto\n$out\n")
        }
        homeButton.setOnClickListener { send("home\n") }

        try {
            bluetoothManager = getSystemService(BLUETOOTH_SERVICE) as BluetoothManager
            if (!bluetoothManager.adapter.isEnabled) bluetoothManager.adapter.enable()
            while (true) if (bluetoothManager.adapter.isEnabled) break
            device = bluetoothManager.adapter.getRemoteDevice("00:18:E4:40:00:06")
            socket =
                device.createRfcommSocketToServiceRecord(UUID.fromString("00001101-0000-1000-8000-00805F9B34FB"))
            Thread {
                while (true) {
                    if (socket.isConnected) {
                        var string = ""
                        if (upButton.isPressed and !downButton.isPressed) string = "u"
                        else if (downButton.isPressed and !upButton.isPressed) string = "d"
                        else if (leftButton.isPressed and !rightButton.isPressed) string = "l"
                        else if (rightButton.isPressed and !leftButton.isPressed) string = "r"

                        if (string != "") send(string + '\n')
                    }
                    Thread.sleep(200)
                }
            }.start()
        } catch (e: SecurityException) {
            showToast(getString(R.string.no_permissions))
        }
    }

    private fun showToast(text: String) {
        runOnUiThread { Toast.makeText(this, text, Toast.LENGTH_SHORT).show() }
    }

    private fun send(text: String) {
        if (socket.isConnected) socket.outputStream.write(text.toByteArray())
    }
}

9


